# -*- coding: utf-8 -*-

import datetime
import re

import jinja2
import lxml
import werkzeug

from .const import OAUTH_BASE_URL, TEST, LOCAL
from .compat import request

class TemplateEngine(object):
    default_context = {
        'TEST': TEST,
        'OAUTH_BASE_URL': OAUTH_BASE_URL,
        'datetime': datetime,
        're': re,
    }

    def __init__(self, search_path, context=None):
        self.env = jinja2.Environment(loader=jinja2.FileSystemLoader(search_path), autoescape=True)
        self.env.add_extension('jinja2.ext.i18n')
        self.env.globals.update(**self.default_context)
        if context:
            self.env.globals.update(context)

        def rn2br(value):
            r = '<br />'
            value = werkzeug.utils.escape(value)
            return value.replace('\r\n', r).replace('\n\r', r).replace('\r', r).replace('\n', r)
        def cleanurl(url):
            if url and not url.startswith(('http://', 'https://')):
                url = 'http://' + url
            return url
        self.env.filters['rn2br'] = rn2br
        self.env.filters['cleanurl'] = cleanurl

        def striphtml(value):
            try:
                tree = lxml.html.document_fromstring(value)
                tree = lxml.html.clean.clean_html(tree)
                return tree.text_content()
            except Exception:
                return value
        self.env.filters['striphtml'] = striphtml

    def _url(self, httprequest, path):
        splithost = httprequest.host.split('.')
        key = path.strip('/').split('/')[0]
        if len(splithost) != 1:
            if LOCAL:
                port = dict(partners=8369, apps=8080).get(key, 8069)
                splithost[-1] = splithost[-1].split(':')[0]
                url = '%s://%s:%s%s' % (httprequest.scheme, '.'.join(splithost), port, path)
                return url
            subdomain = 'www'
            if key == 'doc':
                subdomain = 'doc'
                path = '/'
            if subdomain != splithost[0]:
                splithost[0] = subdomain
                return '%s://%s%s' % (httprequest.scheme, '.'.join(splithost), path)
        return path

    def render(self, template_name, extra=None):
        def url(path):
            return self._url(request.httprequest, path)

        values = {
            'url': url,
            'database': request.session._db,
        }
        if isinstance(extra, dict):
            values.update(**extra)

        template = self.env.get_template(template_name)
        return template.render(**values)
